package main

import (
	"flag"
	"fmt"
	"log"
	"os"
	"os/signal"
	"runtime"
	"syscall"

	"github.com/azoic/wormhole-server/internal/server"
)

var (
	version   = "v1.1.0"
	buildTime = "unknown"
	gitCommit = "unknown"
)

func main() {
	configPath := flag.String("config", "configs/server.yaml", "Configuration file path")
	showVersion := flag.Bool("version", false, "Show version information")
	verboseVersion := flag.Bool("version-verbose", false, "Show detailed version information")
	flag.Parse()

	if *showVersion || *verboseVersion {
		showVersionInfo(*verboseVersion)
		os.Exit(0)
	}

	fmt.Printf("🚀 Starting Wormhole SOCKS5 Server %s\n", version)
	fmt.Printf("📄 Config: %s\n", *configPath)

	// 创建服务器实例
	srv := server.NewServer()

	// 设置信号处理
	sigChan := make(chan os.Signal, 1)
	signal.Notify(sigChan, syscall.SIGINT, syscall.SIGTERM)

	// 启动服务器
	go func() {
		if err := srv.Start(*configPath); err != nil {
			log.Fatalf("Server failed to start: %v", err)
		}
	}()

	// 等待信号
	sig := <-sigChan
	fmt.Printf("\n🛑 Received signal: %v\n", sig)
	fmt.Println("🔄 Shutting down gracefully...")

	// 优雅关闭
	if err := srv.Stop(); err != nil {
		log.Printf("Error during shutdown: %v", err)
	}

	fmt.Println("✅ Server stopped")
}

func showVersionInfo(verbose bool) {
	fmt.Printf("Wormhole SOCKS5 Server %s\n", version)

	if verbose {
		fmt.Printf("\nBuild Information:\n")
		fmt.Printf("  Version:     %s\n", version)
		fmt.Printf("  Build Time:  %s\n", buildTime)
		fmt.Printf("  Git Commit:  %s\n", gitCommit)
		fmt.Printf("  Go Version:  %s\n", runtime.Version())
		fmt.Printf("  Platform:    %s/%s\n", runtime.GOOS, runtime.GOARCH)
		fmt.Printf("  Compiler:    %s\n", runtime.Compiler)

		fmt.Printf("\nFeatures:\n")
		fmt.Printf("  ✅ SOCKS5 Protocol Support\n")
		fmt.Printf("  ✅ Rate Limiting (Token Bucket)\n")
		fmt.Printf("  ✅ Connection Pool Management\n")
		fmt.Printf("  ✅ Memory Optimization\n")
		fmt.Printf("  ✅ Health Check & Metrics\n")
		fmt.Printf("  ⚠️  DNS Caching (Temporarily Disabled)\n")
		fmt.Printf("  ⚠️  Transparent Proxy (Default Disabled)\n")
	} else {
		fmt.Printf("Build time: %s\n", buildTime)
	}
}
